/*!
 * \file    CFreqDictionary.cpp
 * \brief   Implementation of CFreqDictionary
 * \author  shinyak
 * 
 * $Id$
 * Copyright (C) 2005- Shinya KAWANAKA All Rights Reserved.
 */

#include "StdAfx.h"
#include "CFreqDictionary.h"
#include "Utility.h"

const CString CFreqDictionaryManager::s_dictFilename = 
    GetCurrentDirectorySafely() + _T("dict.dct");

CFreqDictionaryManager::CFreqDictionaryManager(void)
{
    m_allFreq = 0;
}

CFreqDictionaryManager::~CFreqDictionaryManager(void)
{
}

CFreqDictionaryManager& CFreqDictionaryManager::Instance()
{
    static CFreqDictionaryManager s_manager;
    return s_manager;
}

void CFreqDictionaryManager::LoadDictionary()
{
    m_freq.clear();

    // CStd
    FILE* fp = _tfopen(s_dictFilename, _T("r"));
    if (fp == NULL) { return; }
    
    int x;
    TCHAR str[80];

    while (_ftscanf(fp, _T("%d,%79s"), &x, str) == 2) {
        m_freq[str] = x;
        m_allFreq += x;
    }

    fclose(fp);
}

void CFreqDictionaryManager::SaveDictionary() const
{
    CStdioFile file;
    CFileException exception;
    if (!file.Open(s_dictFilename, CFile::modeCreate | CFile::modeWrite, &exception)) {
        TCHAR why[256];
        exception.GetErrorMessage(why, 256);
        AfxMessageBox(why);
        return;
    }

    for (std::map<CString, int>::const_iterator it = m_freq.begin(); it != m_freq.end(); ++it) {
        CString str = StringF(_T("%d,%s\n"), it->second, static_cast<LPCTSTR>(it->first));
        file.WriteString(str);
    }

    file.Close();
}

int CFreqDictionaryManager::GetFrequency(const CString& str) const
{
    std::map<CString, int>::const_iterator it = m_freq.find(str);

    if (it == m_freq.end()) { return 0; }
    else { return it->second; }
}

void CFreqDictionaryManager::UpFrequency(const CString& str)
{
    std::map<CString, int>::iterator it = m_freq.find(str);

    if (it != m_freq.end()) {
        it->second++;
    } else {
        m_freq.insert(std::make_pair(str, 1));
    }
}

void CFreqDictionaryManager::DownFrequency(const CString& str)
{
    std::map<CString, int>::iterator it = m_freq.find(str);

    if (it != m_freq.end() && it->second > 0) {
        it->second--;
    }
}

