/*!
 * \file    CSharkKeyboard.cpp
 * \brief   Implementation of CSharkKeyboard.cpp
 * \author  shinyak
 *
 * $Id$
 * Copyright (C) 2005- Shinya KAWANAKA All Rights Reserved.
 */

//
// yutaochi ϐ̓NX̐擪AƂ̂܂Ă̂ŁA
// R[fBOK̗pĂ݂Bʂ̓NX̍ŌゾƎvB
// g݂zƂẮA܂OɎčsCȂ͂Ȃ
// vB
//

#include "StdAfx.h"
#include "JShark.h"
#include "CSharkKeyboard.h"
#include "Utility.h"
#include "CDebugNew.h"
#include "CFreqDictionary.h"
class CSharkKey;

//----------------------------------------------------------------------
// 萔錾

/*!
 * \brief L[̔a
 */
const int CSHARK_KEY_RADIUS = 32;

/*!
 * _ÃL[ǂꂾĂ邩킷B
 */
enum NearFarType {
    SHARK_NEAR,
    SHARK_MIDDLE,
    SHARK_FAR,
};

struct KeyTable {
    TCHAR key;          //!< Key
    TCHAR keyShift;
    TCHAR wVk;          //!< UNICODE ̂Ƃ 0AȊO VK_* 
    TCHAR wVkShift;
    int   centerX;      //!< L[S X W
    int   centerY;      //!< L[S Y W
};

KeyTable KEY_TABLE[] = 
{
    { _T(''), _T(''),   0, 0, CSHARK_KEY_RADIUS    , CSHARK_KEY_RADIUS      }, 
    { _T(''), _T(''),   0, 0, CSHARK_KEY_RADIUS    , CSHARK_KEY_RADIUS * 3  }, 
    { _T(''), _T(''),   0, 0, CSHARK_KEY_RADIUS    , CSHARK_KEY_RADIUS * 5  }, 
    { _T(''), _T(''),   0, 0, CSHARK_KEY_RADIUS    , CSHARK_KEY_RADIUS * 7  }, 
    { _T(''), _T(''),   0, 0, CSHARK_KEY_RADIUS    , CSHARK_KEY_RADIUS * 9  }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS    , CSHARK_KEY_RADIUS * 11 }, 

    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 3, CSHARK_KEY_RADIUS * 2  }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 3, CSHARK_KEY_RADIUS * 4  }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 3, CSHARK_KEY_RADIUS * 6  }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 3, CSHARK_KEY_RADIUS * 8  }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 3, CSHARK_KEY_RADIUS * 10 }, 
    { _T(''), _T(''),   0, 0, CSHARK_KEY_RADIUS * 3, CSHARK_KEY_RADIUS * 12 }, 

    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 5, CSHARK_KEY_RADIUS * 1 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 5, CSHARK_KEY_RADIUS * 3 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 5, CSHARK_KEY_RADIUS * 5 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 5, CSHARK_KEY_RADIUS * 7 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 5, CSHARK_KEY_RADIUS * 9 }, 
    { _T(''), _T(''),   0, 0, CSHARK_KEY_RADIUS * 5, CSHARK_KEY_RADIUS * 11 }, 

    { _T(''), _T(''),   0, 0, CSHARK_KEY_RADIUS * 7, CSHARK_KEY_RADIUS * 2 }, 
    { _T(''), _T(''),   0, 0, CSHARK_KEY_RADIUS * 7, CSHARK_KEY_RADIUS * 4 }, 
    { _T(''), _T(''),   0, 0, CSHARK_KEY_RADIUS * 7, CSHARK_KEY_RADIUS * 6 }, 
    { _T(''), _T(''),   0, 0, CSHARK_KEY_RADIUS * 7, CSHARK_KEY_RADIUS * 8 }, 
    { _T(''), _T(''),   0, 0, CSHARK_KEY_RADIUS * 7, CSHARK_KEY_RADIUS * 10 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 7, CSHARK_KEY_RADIUS * 12 }, 

    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 9, CSHARK_KEY_RADIUS * 1 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 9, CSHARK_KEY_RADIUS * 3 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 9, CSHARK_KEY_RADIUS * 5 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 9, CSHARK_KEY_RADIUS * 7 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 9, CSHARK_KEY_RADIUS * 9 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 9, CSHARK_KEY_RADIUS * 11 }, 

    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 11, CSHARK_KEY_RADIUS * 2 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 11, CSHARK_KEY_RADIUS * 4 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 11, CSHARK_KEY_RADIUS * 6 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 11, CSHARK_KEY_RADIUS * 8 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 11, CSHARK_KEY_RADIUS * 10 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 11, CSHARK_KEY_RADIUS * 12 }, 

    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 13, CSHARK_KEY_RADIUS * 1 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 13, CSHARK_KEY_RADIUS * 3 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 13, CSHARK_KEY_RADIUS * 5 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 13, CSHARK_KEY_RADIUS * 7 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 13, CSHARK_KEY_RADIUS * 9 }, 
    { _T('A'), _T('B'), 0, 0, CSHARK_KEY_RADIUS * 13, CSHARK_KEY_RADIUS * 11 }, 

    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 15, CSHARK_KEY_RADIUS * 2 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 15, CSHARK_KEY_RADIUS * 4 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 15, CSHARK_KEY_RADIUS * 6 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 15, CSHARK_KEY_RADIUS * 8 }, 
    { _T(''), _T(''), 0, 0, CSHARK_KEY_RADIUS * 15, CSHARK_KEY_RADIUS * 10 }, 
    { _T(''), _T('['), 0, 0, CSHARK_KEY_RADIUS * 15, CSHARK_KEY_RADIUS * 12 }, 

    { _T(''), _T(''), VK_BACK,   0, CSHARK_KEY_RADIUS * 19, CSHARK_KEY_RADIUS * 2 }, 
    { _T(''), _T(''), VK_DELETE, 0, CSHARK_KEY_RADIUS * 19, CSHARK_KEY_RADIUS * 4 }, 
    { _T(''), _T(''), VK_RETURN, 0, CSHARK_KEY_RADIUS * 21, CSHARK_KEY_RADIUS * 1 }, 
    { _T(''), _T(''), VK_SPACE,  0, CSHARK_KEY_RADIUS * 21, CSHARK_KEY_RADIUS * 3 }, 

    { _T(''), _T(''), VK_UP,    0, CSHARK_KEY_RADIUS * 21, CSHARK_KEY_RADIUS * 7 }, 
    { _T(''), _T(''), VK_LEFT,  0, CSHARK_KEY_RADIUS * 19, CSHARK_KEY_RADIUS * 8 }, 
    { _T(''), _T(''), VK_RIGHT, 0, CSHARK_KEY_RADIUS * 23, CSHARK_KEY_RADIUS * 8 }, 
    { _T(''), _T(''), VK_DOWN,  0, CSHARK_KEY_RADIUS * 21, CSHARK_KEY_RADIUS * 9 }, 

};

const CPoint SHAEK_KEY_OOPS_CENTER(CSHARK_KEY_RADIUS * 21, CSHARK_KEY_RADIUS * 12);

//----------------------------------------------------------------------

/*!
 * \brief   L[̃Cxg\
 */
struct SharkEvent
{
    const CSharkKey*    pKey;               //!< 
    bool                shift;              //!< VtgĂ邩
    double              cert;               //!< M

    SharkEvent(const CSharkKey* pKey, bool shift, double cert)
        : pKey(pKey)
        , shift(shift)
        , cert(cert)
    {
    }
};

//----------------------------------------------------------------------

/*!
 * \brief   Shark ̒ʏ̃L[\
 */
class CSharkKey
{
protected:
    CPoint          m_center;
    TCHAR           m_key;          //!< Key
    TCHAR           m_shiftKey;     //!< ShiftKey

public:
    static const int KEY_RADIUS = CSHARK_KEY_RADIUS;

public:
    CSharkKey();
    CSharkKey(TCHAR key, TCHAR shiftKey, const CPoint& center);

    virtual ~CSharkKey(void);

public:
    virtual void SendKeyboardMessage(bool shift) const;

public:
    virtual void Draw(CDC* pDC) const;

    /*!
     * \brief   point ÃL[ɔׂĉ߂ԂB
     * \note    Distance  SHARK_NEAR ԂƂốA̃L[݂̂łKv ()
     */
    virtual NearFarType Distance(const CPoint& point) const;

public:
    const CPoint& GetCenter() const { return m_center; }
    TCHAR GetKey() const { return m_key; }
    TCHAR GetKeyShift() const { return m_shiftKey; }
};

//----------------------------------------------------------------------

/*!
 * \brief   ûȂvL[
 */
class CSharkKeyOops : public CSharkKey
{
public:
    CSharkKeyOops(const CPoint& center);
    ~CSharkKeyOops(void);

public:

    virtual void SendKeyboardMessage(bool shift) const;

    virtual void Draw(CDC* pDC) const;

    virtual NearFarType Distance(const CPoint& point) const;
};


//----------------------------------------------------------------------

class CSharkKeyboardManagerImpl
{
    typedef std::map<TCHAR, INPUT*>                 MappingType;
    typedef std::vector<CSharkKey*>                 KeyType;
    typedef std::vector<SharkEvent>                 EventSequenceType;

    bool                m_autoSpace;
    bool                m_useFreq;
    bool                m_recFreq;
    CString             m_prevString;
    MappingType         m_mapping;
    KeyType             m_keys;
    EventSequenceType   m_eventSequence; 

public:
    CSharkKeyboardManagerImpl();
    ~CSharkKeyboardManagerImpl();

private:

    bool IsOopsKey(const CSharkKey* pKey) const;

    CString StringOfEvent(const EventSequenceType& events);

    void Initialize();

    /*!
     * \brief   point ŕ\_ɁAƂ߂L[擾B
     */
    const CSharkKey* GetMostNearKey(const CPoint& point) const;

    bool HasIntersection(const std::vector<CPoint>& ps) const;

    void Smoothing(const std::vector<CPoint>& orbit, std::vector<CPoint>& newOrbit) const;


    void FindEvents(const std::vector<CPoint>& orbit, std::vector<std::vector<SharkEvent> >& events) const;


    void FindMostReliableEventSequence(const std::vector<std::vector<SharkEvent> >& events,
                                       size_t nth,
                                       double seqCert,
                                       EventSequenceType& sequence,
                                       double& maxCert, 
                                       EventSequenceType& result);
public:
    void SetAutoSpace(bool autoSpace);
    bool GetAutoSpace() const;

public:
    void SetUseFrequency(bool useFreq);
    bool GetUseFrequency() const;

public:
    void SetRecFrequency(bool recFreq);
    bool GetRecFrequency() const;

public:
    void DrawKeyboard(CDC* pDC) const;

    /*!
     * \brief   OՂFāA SendInput() ŃL[{[h𑗂邱Ƃo悤ɂB
     */
    void Recognize(const std::vector<CPoint>& orbit);

    void GetINPUTOfKey(TCHAR letter, int& outN, INPUT*& outInput);

    /*!
     * \brief   ȑO Recognize ɊÂăL[{[hG~[VsB
     */
    void SendInput();
};

//----------------------------------------------------------------------


CSharkKeyboardManager::CSharkKeyboardManager()
    : m_pImpl(new CSharkKeyboardManagerImpl)
{
}


CSharkKeyboardManager::~CSharkKeyboardManager()
{
}


CSharkKeyboardManager& CSharkKeyboardManager::Instance()
{
    static CSharkKeyboardManager s_manager;
    return s_manager;
}


void CSharkKeyboardManager::DrawKeyboard(CDC* pDC) const
{
    m_pImpl->DrawKeyboard(pDC);
}


void CSharkKeyboardManager::Recognize(const std::vector<CPoint>& orbit)
{
    m_pImpl->Recognize(orbit);
}


void CSharkKeyboardManager::SendInput()
{
    m_pImpl->SendInput();
}


void CSharkKeyboardManager::GetINPUTOfKey(TCHAR letter, int& outN, INPUT*& outInput)
{
    m_pImpl->GetINPUTOfKey(letter, outN, outInput);
}


void CSharkKeyboardManager::SetAutoSpace(bool autoSpace)
{
    m_pImpl->SetAutoSpace(autoSpace);
}


bool CSharkKeyboardManager::GetAutoSpace() const
{
    return m_pImpl->GetAutoSpace();
}


void CSharkKeyboardManager::SetUseFrequency(bool useFreq)
{
    m_pImpl->SetUseFrequency(useFreq);
}


bool CSharkKeyboardManager::GetUseFrequency() const
{
    return m_pImpl->GetUseFrequency();
}


void CSharkKeyboardManager::SetRecFrequency(bool recFreq)
{
    m_pImpl->SetRecFrequency(recFreq);
}


bool CSharkKeyboardManager::GetRecFrequency() const
{
    return m_pImpl->GetRecFrequency();
}


//----------------------------------------------------------------------

CSharkKey::CSharkKey()
{
}


CSharkKey::CSharkKey(TCHAR key, TCHAR shiftKey, const CPoint& center)
    : m_key(key)
    , m_shiftKey(shiftKey)
    , m_center(center)
{
}


CSharkKey::~CSharkKey(void)
{
}


void CSharkKey::SendKeyboardMessage(bool shift) const
{
    int n = 0;
    INPUT* pInput = NULL;
    CSharkKeyboardManager::Instance().GetINPUTOfKey(shift ? GetKeyShift() : GetKey(), n, pInput);

    SendInput(n, pInput, sizeof(INPUT));
}


void CSharkKey::Draw(CDC* pDC) const
{
    // Zp``
    CPoint points[6];
    for (int i = 0; i < 6; ++i) {
        points[i].x = m_center.x + static_cast<int>(KEY_RADIUS * cos(2 * M_PI * i / 6));
        points[i].y = m_center.y + static_cast<int>(KEY_RADIUS * sin(2 * M_PI * i / 6));
    }
    pDC->Polygon(points, 6);

    if (m_key != _T('')) {
        pDC->TextOut(m_center.x - KEY_RADIUS / 2, m_center.y - KEY_RADIUS / 2, CString(m_key));
    }
    if (m_shiftKey != _T('')) { pDC->TextOut(m_center.x, m_center.y, CString(m_shiftKey)); } 
}


NearFarType CSharkKey::Distance(const CPoint& point) const
{
    const double x = point.x - m_center.x;
    const double y = point.y - m_center.y;
    double distance = sqrt(x * x + y * y);
    if (distance < KEY_RADIUS) { return SHARK_NEAR; }
    if (distance < 3 * KEY_RADIUS / 2) { return SHARK_MIDDLE; }
    return SHARK_FAR;
}


//----------------------------------------------------------------------

CSharkKeyOops::CSharkKeyOops(const CPoint& center) : 
    CSharkKey(_T(''), _T(''), center)
{
}
    
CSharkKeyOops::~CSharkKeyOops(void)
{
}


void CSharkKeyOops::SendKeyboardMessage(bool shift) const
{
    ASSERT_NOT_REACHED();
}


void CSharkKeyOops::Draw(CDC* pDC) const
{
    CRect rect(m_center.x - CSHARK_KEY_RADIUS * 5 / 2,
                m_center.y - CSHARK_KEY_RADIUS,
                m_center.x + CSHARK_KEY_RADIUS * 5 / 2,
                m_center.y + CSHARK_KEY_RADIUS);
    pDC->Rectangle(rect);

    pDC->TextOut(m_center.x - 36, m_center.y - 12, _T("̂ȂI"));
}


NearFarType CSharkKeyOops::Distance(const CPoint& point) const
{
    CRect rectNear(m_center.x - CSHARK_KEY_RADIUS * 5 / 2,
                    m_center.y - CSHARK_KEY_RADIUS,
                    m_center.x + CSHARK_KEY_RADIUS * 5 / 2,
                    m_center.y + CSHARK_KEY_RADIUS);
    if (rectNear.left <= point.x && point.x <= rectNear.right &&
        rectNear.top  <= point.y && point.y <= rectNear.bottom)
    {
        return SHARK_NEAR;
    }

    CRect rectMiddle(m_center.x - CSHARK_KEY_RADIUS * 7 / 2,
                        m_center.y - CSHARK_KEY_RADIUS * 3 / 2,
                        m_center.x + CSHARK_KEY_RADIUS * 7 / 2,
                        m_center.y + CSHARK_KEY_RADIUS * 3 / 2);
    if (rectMiddle.left <= point.x && point.x <= rectMiddle.right &&
        rectMiddle.top  <= point.y && point.y <= rectMiddle.bottom)
    {
        return SHARK_NEAR;
    }

    return SHARK_FAR;
}


//----------------------------------------------------------------------

CSharkKeyboardManagerImpl::CSharkKeyboardManagerImpl()
    : m_autoSpace(false)
    , m_useFreq(true)
    , m_recFreq(true)
{
    Initialize();
}

CSharkKeyboardManagerImpl::~CSharkKeyboardManagerImpl()
{
    std::for_each(m_keys.begin(), m_keys.end(), delete_ptr());
    for (MappingType::iterator it = m_mapping.begin(); it != m_mapping.end(); ++it) {
        delete[] it->second;
    }
}

bool CSharkKeyboardManagerImpl::IsOopsKey(const CSharkKey* pKey) const
{
    if (pKey == NULL) { return false; }
    return dynamic_cast<const CSharkKeyOops*>(pKey) != NULL;
}

CString CSharkKeyboardManagerImpl::StringOfEvent(const EventSequenceType& events)
{
    CString dict;

    for (size_t i = 0; i < events.size(); ++i) {
        if (events[i].shift && events[i].pKey->GetKeyShift() != _T('')) {
            dict += events[i].pKey->GetKeyShift();
        } else if (events[i].pKey->GetKey() != _T('')) {
            dict += events[i].pKey->GetKey();
        }
    }

    return dict;
}
    
void CSharkKeyboardManagerImpl::Initialize()
{
    size_t KEYTABLE_NUM = sizeof(KEY_TABLE) / sizeof(KEY_TABLE[0]);
    m_keys.reserve(KEYTABLE_NUM + 1);

    // ------ KeyTable ̍쐬
    // ̂Ȃ{^Aʂ̃L[
    m_keys.push_back(new CSharkKeyOops(SHAEK_KEY_OOPS_CENTER));
    for (int i = 0; i < static_cast<int>(KEYTABLE_NUM); ++i) {
        m_keys.push_back(new CSharkKey(KEY_TABLE[i].key, KEY_TABLE[i].keyShift,
                                        CPoint(KEY_TABLE[i].centerX, KEY_TABLE[i].centerY)));
    }

    // ------ INPUT 쐬
    for (int i = 0; i < static_cast<int>(KEYTABLE_NUM); ++i) {
        // unshift version
        if (KEY_TABLE[i].key != _T('')) {
            INPUT* pInput = new INPUT[2];
            pInput[0].type = pInput[1].type = INPUT_KEYBOARD;
            if (KEY_TABLE[i].wVk == 0) { // 0 ̂Ƃ UNICODE ƒ߂
                pInput[0].ki.wVk     = pInput[1].ki.wVk = 0;
                pInput[0].ki.wScan   = pInput[1].ki.wScan = KEY_TABLE[i].key;
                pInput[0].ki.dwFlags = KEYEVENTF_UNICODE;
                pInput[1].ki.dwFlags = KEYEVENTF_KEYUP | KEYEVENTF_UNICODE;
            } else {
                pInput[0].ki.wVk     = pInput[1].ki.wVk = KEY_TABLE[i].wVk;
                pInput[0].ki.wScan   = pInput[1].ki.wScan = static_cast<WORD>(MapVirtualKey(KEY_TABLE[i].wVk, 0));
                pInput[0].ki.dwFlags = 0;
                pInput[1].ki.dwFlags = KEYEVENTF_KEYUP;
            }
            pInput[0].ki.time = pInput[1].ki.time = 0;
            pInput[0].ki.dwExtraInfo = pInput[1].ki.dwExtraInfo = 0;

            BOOST_ASSERT(m_mapping.find(KEY_TABLE[i].key) == m_mapping.end());
            m_mapping.insert(std::make_pair(KEY_TABLE[i].key, pInput));

        }
        // shift version
        if (KEY_TABLE[i].keyShift != _T('')) {
            INPUT* pInput = new INPUT[2];
            pInput[0].type = pInput[1].type = INPUT_KEYBOARD;
            if (KEY_TABLE[i].wVkShift == 0) { // 0 ̂Ƃ UNICODE ƒ߂
                pInput[0].ki.wVk     = pInput[1].ki.wVk = 0;
                pInput[0].ki.wScan   = pInput[1].ki.wScan = KEY_TABLE[i].keyShift;
                pInput[0].ki.dwFlags = KEYEVENTF_UNICODE;
                pInput[1].ki.dwFlags = KEYEVENTF_KEYUP | KEYEVENTF_UNICODE;
            } else {
                pInput[0].ki.wVk     = pInput[1].ki.wVk = KEY_TABLE[i].wVkShift;
                pInput[0].ki.wScan   = pInput[1].ki.wScan = static_cast<WORD>(MapVirtualKey(KEY_TABLE[i].wVkShift, 0));
                pInput[0].ki.dwFlags = 0;
                pInput[1].ki.dwFlags = KEYEVENTF_KEYUP;
            }
            pInput[0].ki.time = pInput[1].ki.time = 0;
            pInput[0].ki.dwExtraInfo = pInput[1].ki.dwExtraInfo = 0;

            BOOST_ASSERT(m_mapping.find(KEY_TABLE[i].keyShift) == m_mapping.end());
            m_mapping.insert(std::make_pair(KEY_TABLE[i].keyShift, pInput));
        }
    }
}


const CSharkKey* CSharkKeyboardManagerImpl::GetMostNearKey(const CPoint& point) const
{
    double maxDist = std::numeric_limits<double>::max();
    const CSharkKey* pKey = NULL;

    for (KeyType::const_iterator it = m_keys.begin(); it != m_keys.end(); ++it) {
        double x = point.x - (*it)->GetCenter().x;
        double y = point.y - (*it)->GetCenter().y;
        double dist = sqrt(x * x + y * y);
        if (dist < maxDist) {
            maxDist = dist;
            pKey    = *it;
        }
    }

    return pKey;
}


void CSharkKeyboardManagerImpl::Smoothing(const std::vector<CPoint>& orbit, std::vector<CPoint>& newOrbit) const
{
    typedef std::vector<CPoint>::const_iterator ConstIter;

    newOrbit.clear();
    newOrbit.reserve(orbit.size() + 2);

    // ------ ܂Aorbit  V[vɂA炩ɂB
    // 1. {IɁAQ_Ԃ̕ς邱Ƃ smoothing
    // 2. px}ɕςĂƎvꍇ́AςȂB
    // 3. ŏƍŌ͂̂܂ܓB

    newOrbit.push_back(orbit.front());
    ConstIter prev    = orbit.begin();
    ConstIter current = prev + 1;
    ConstIter future  = current;
    while (++future != orbit.end()) {
        // ܂߂͍̗̂pȂB
        double x = future->x - current->x;
        double y = future->y - current->y;
        if (sqrt(x * x + y * y) < 4) { continue; }

        double prevAngle = atan2(static_cast<double>(current->y - prev->y), current->x - current->x);
        double currAngle = atan2(static_cast<double>(future->y - current->y), future->x - current->x);
        double diffAngle = RoundDifferencial(prevAngle, currAngle);
        if (fabs(diffAngle) < M_PI / 6) {
            newOrbit.push_back(CPoint((prev->x + future->x) / 2, (prev->y + future->y) / 2));
        } else {
            newOrbit.push_back(*current);
        }

        prev = current;
        current = future;
    }
    newOrbit.push_back(orbit.back());
}

bool CSharkKeyboardManagerImpl::HasIntersection(const std::vector<CPoint>& ps) const
{
    // ------ BɁAx͂ȂĂȂ΁AƂ݂ȂȂB

    std::vector<double> dist;
    dist.reserve(ps.size());

    // ̕ai[
    double distSum = 0.0;
    for (int i = 1; i < static_cast<int>(ps.size()); ++i) {
        double x = ps[i].x - ps[i - 1].x;
        double y = ps[i].y - ps[i - 1].y;
        distSum += sqrt(x * x + y * y);
        dist.push_back(distSum);
    }

    for (int i = 1; i < static_cast<int>(ps.size()); ++i) {
        for (int j = 1; j < i - 2; ++j) {
            // ߂ƔfꍇAƂ݂ȂȂBȗB
            if (dist[i - 1] - dist[j - 1] < CSHARK_KEY_RADIUS / 2) { continue; }
            // 
            if (IsLineIntersection(ps[j - 1], ps[j], ps[i - 1], ps[i])) {
                return true;
            }
        }
    }

    return false;
}


void CSharkKeyboardManagerImpl::FindEvents(const std::vector<CPoint>& orbit, std::vector<std::vector<SharkEvent> >& events) const
{
    const double INTERSECTION_CERT  = 1.00;
    const double REMARK_NEAR_CERT   = 1.00;
    const double REMARK_MIDDLE_CERT = 0.70;
    const double NOTHING_CERT       = 0.30;
    const double INTER_NOTHING_CERT = 0.05;

    BOOST_ASSERT(orbit.size() >= 2);

    events.clear();

    // orbit ͂ACxgɒBCxg͎̎ނ
    // CxǵA̃L[ SHARK_NEAR 𓮂ԂɁAXPȂB
    // i.e. Qȏł΁AxL[̊OɏoKvB

    // 1. ŏCxgAŌCxg
    // 2. px̍Cxg
    // 3. L[ŌCxg

    double diffSum = 0.0;               // ̘a
    std::vector<CPoint>     ps;         // CxgŎg

    double prvX = static_cast<double>(orbit[1].x - orbit[0].x);
    double prvY = static_cast<double>(orbit[1].y - orbit[0].y);
    double prvD = sqrt(prvX * prvX + prvY * prvY);
    double prvR = atan2(prvY, prvX);
    const CSharkKey* pPrvKey = GetMostNearKey(orbit[0]);
    if (pPrvKey->Distance(orbit[0]) == SHARK_FAR) { pPrvKey = NULL; }
    
    diffSum += prvD;
    ps.push_back(orbit[0]);
    ps.push_back(orbit[1]);

    std::vector<SharkEvent> curEvents;

    if (pPrvKey != NULL) {
        curEvents.push_back(SharkEvent(pPrvKey, false, REMARK_NEAR_CERT));
    }

    for (size_t i = 2; i < orbit.size(); ++i) {
        double curX = static_cast<double>(orbit[i].x - orbit[i - 1].x);
        double curY = static_cast<double>(orbit[i].y - orbit[i - 1].y);
        double curD = sqrt(curX * curX + curY * curY);
        double curR = atan2(curY, curX);

        //TODO: pPrkKey gčo
        const CSharkKey* pKey = GetMostNearKey(orbit[i]);
        if (pKey->Distance(orbit[i]) == SHARK_FAR) { pKey = NULL; }
        ps.push_back(orbit[i]);

        diffSum += curD;
        double r = RoundDifferencial(prvR, curR);

        // Cxg`FbN 
        if (pPrvKey != NULL && fabs(r) > M_PI / 6) {
            // Cxgō\BǉB
            curEvents.clear();
            for (KeyType::const_iterator it = m_keys.begin(); it != m_keys.end(); ++it) {
                switch ((*it)->Distance(orbit[i])) {
                case SHARK_NEAR:
                    curEvents.push_back(SharkEvent(*it, false, REMARK_NEAR_CERT));
                    break;
                case SHARK_MIDDLE:
                    curEvents.push_back(SharkEvent(*it, false, REMARK_MIDDLE_CERT));
                    break;
                case SHARK_FAR:
                    break;
                default:
                    ASSERT_NOT_REACHED();
                    break;
                }
            }
            curEvents.push_back(SharkEvent(NULL, false, NOTHING_CERT));
            diffSum = 0;
        }

        if (pPrvKey != pKey) {
            // Cxg`FbN
            if (pPrvKey != NULL && HasIntersection(ps)) {
                curEvents.clear();
                curEvents.push_back(SharkEvent(pPrvKey, true, INTERSECTION_CERT));
                curEvents.push_back(SharkEvent(NULL, true, INTER_NOTHING_CERT));
            }
            ps.clear();
            ps.push_back(orbit[i - 1]);
            ps.push_back(orbit[i]);

            // Cxg΁ACxg push_back
            if (!curEvents.empty()) {
                events.push_back(curEvents);
            }
            curEvents.clear();
        }
        
        prvX = curX;
        prvY = curY;
        prvD = curD;
        prvR = curR;
        pPrvKey = pKey;
    }

    // ŌɂCxg`FbNACxg΁ACxg push_back
    // CxgȂ΁AŏI_ƂāAڃCxgݒ
    if (pPrvKey != NULL && HasIntersection(ps)) {
        curEvents.clear();
        curEvents.push_back(SharkEvent(pPrvKey, true, INTERSECTION_CERT));
        curEvents.push_back(SharkEvent(NULL, true, INTER_NOTHING_CERT));
    }

    if (!curEvents.empty()) {
        events.push_back(curEvents);
    } else if (pPrvKey != NULL) {
        curEvents.clear();
        for (KeyType::const_iterator it = m_keys.begin(); it != m_keys.end(); ++it) {
            switch ((*it)->Distance(orbit.back())) {
            case SHARK_NEAR:
                curEvents.push_back(SharkEvent(*it, false, REMARK_NEAR_CERT));
                break;
            case SHARK_MIDDLE:
                curEvents.push_back(SharkEvent(*it, false, REMARK_MIDDLE_CERT));
                break;
            case SHARK_FAR:
                break;
            default:
                ASSERT_NOT_REACHED();
                break;
            }
        }
        curEvents.push_back(SharkEvent(NULL, false, NOTHING_CERT));
        events.push_back(curEvents);
    }   

}


void CSharkKeyboardManagerImpl::FindMostReliableEventSequence(
    const std::vector<std::vector<SharkEvent> >& events,
    size_t nth,
    double seqCert,
    EventSequenceType& sequence,
    double& maxCert, 
    EventSequenceType& result)
{
    // TODO: STxBƍlB

    // ċÅ
    if (nth == events.size()) {
        double d = 1.0;
        if (GetUseFrequency()) {
            CString str = StringOfEvent(sequence);
            int h = CFreqDictionaryManager::Instance().GetFrequency(str);
            d = log(1.0 + h) + 1.0;
        }

        if (seqCert * d > maxCert) {
            maxCert = seqCert * d;
            result = sequence;
        }
        return;
    }

    // ċA
    for (size_t i = 0; i < events[nth].size(); ++i) {
        double newSeqCert = seqCert * events[nth][i].cert;
        if (events[nth][i].pKey != NULL) {
            sequence.push_back(events[nth][i]);
            FindMostReliableEventSequence(events, nth + 1, newSeqCert, sequence, maxCert, result);
            sequence.pop_back();
        } else {
            FindMostReliableEventSequence(events, nth + 1, newSeqCert, sequence, maxCert, result);
        }
    }
}


void CSharkKeyboardManagerImpl::SetAutoSpace(bool autoSpace)
{
    m_autoSpace = autoSpace;
}


bool CSharkKeyboardManagerImpl::GetAutoSpace() const
{
    return m_autoSpace;
}

void CSharkKeyboardManagerImpl::SetUseFrequency(bool useFreq)
{
    m_useFreq = useFreq;
}


bool CSharkKeyboardManagerImpl::GetUseFrequency() const
{
    return m_useFreq;
}


void CSharkKeyboardManagerImpl::SetRecFrequency(bool recFreq)
{
    m_recFreq = recFreq;
}


bool CSharkKeyboardManagerImpl::GetRecFrequency() const
{
    return m_recFreq;
}


void CSharkKeyboardManagerImpl::DrawKeyboard(CDC* pDC) const
{
    HGDIOBJ pOldPen   = pDC->SelectObject(GetStockObject(BLACK_PEN));
    HGDIOBJ pOldBrush = pDC->SelectObject(GetStockObject(WHITE_BRUSH));

    for (KeyType::const_iterator it = m_keys.begin(); it != m_keys.end(); ++it) {
        (*it)->Draw(pDC);
    }

    pDC->SelectObject(pOldPen);
    pDC->SelectObject(pOldBrush);
}

void CSharkKeyboardManagerImpl::Recognize(const std::vector<CPoint>& orbit)
{
    // ܂萔Ȃ̂͂B
    if (orbit.size() < 2) { return; }

    // ŌオûȂvL[łÂȂB
    if (IsOopsKey(GetMostNearKey(orbit.back()))) {
        return;
    }

    // ͂𕽊邱ƂŁACxg₷B
    std::vector<CPoint> newOrbit;
    Smoothing(orbit, newOrbit);

    // CxgB
    std::vector<std::vector<SharkEvent> > events;
    FindEvents(newOrbit, events);

    // STāAƂ肻ȕ𓾂B
    EventSequenceType result;
    double maxCert = 0.0;
    FindMostReliableEventSequence(events, 0, 1.0, EventSequenceType(), maxCert, result);

    m_eventSequence = result;

    // pxt@CɓB
    CString dict = StringOfEvent(m_eventSequence);
    m_prevString = dict;

    if (dict.GetLength() >= 2 && GetRecFrequency()) {
        CFreqDictionaryManager::Instance().UpFrequency(dict);
    }
}


void CSharkKeyboardManagerImpl::GetINPUTOfKey(TCHAR letter, int& outN, INPUT*& outInput)
{
    MappingType::const_iterator it = m_mapping.find(letter);
    if (it == m_mapping.end()) {
        outN = 0;
        outInput = NULL;
    } else {
        outN = 2;
        outInput = it->second;
    }
}


void CSharkKeyboardManagerImpl::SendInput()
{
    if (m_eventSequence.empty()) { return; }

    for (size_t i = 0; i < m_eventSequence.size(); ++i) {
        m_eventSequence[i].pKey->SendKeyboardMessage(m_eventSequence[i].shift);
    }

    // auto-space LȂƂ́AVK_SPACE ͂B
    // AP͂̂Ƃ̓Xy[XȂB
    if (GetAutoSpace() && m_eventSequence.size() != 1) {
        INPUT pInput[2];
            pInput[0].type = pInput[1].type = INPUT_KEYBOARD;
        pInput[0].ki.wVk     = pInput[1].ki.wVk = VK_SPACE;
        pInput[0].ki.wScan   = pInput[1].ki.wScan = static_cast<WORD>(MapVirtualKey(VK_SPACE, 0));
        pInput[0].ki.time = pInput[1].ki.time = 0;
        pInput[0].ki.dwExtraInfo = pInput[1].ki.dwExtraInfo = 0;
        pInput[0].ki.dwFlags = 0;
        pInput[1].ki.dwFlags = KEYEVENTF_KEYUP;

        ::SendInput(2, pInput, sizeof(INPUT));
    }

    m_eventSequence.clear();
}
