/*!
 * \file    JShark.cpp
 * \brief   Implementation of JShark class.
 * \author  shinyak
 *
 * $Id$
 * Copyright (C) 2005- Shinya KAWANAKA All Rights Reserved.
 */

#include "StdAfx.h"
#include <locale.h>
#include "JShark.h"
#include "CMainFrame.h"
#include "Resource.h"
#include "CSharkKeyboard.h"
#include "CAboutDialog.h"
#include "CFreqDictionary.h"
#include "CDebugNew.h"

namespace ProjectConfig
{
    //! B CJSharkApp IuWFNgłB
    JSharkApp theApp;
}

JSharkApp* GetJSharkApp()
{
    return &ProjectConfig::theApp;
}

//----------------------------------------------------------------------

BEGIN_MESSAGE_MAP(JSharkApp, CWinApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
    ON_COMMAND(ID_JSHARK_EXIT, OnJSharkExit)
    ON_COMMAND(ID_JSHARK_ABOUT, OnAppAbout)

    ON_COMMAND(ID_JSHARK_AUTOSPACE, OnJSharkAutoSpace)
    ON_UPDATE_COMMAND_UI(ID_JSHARK_AUTOSPACE, OnJSharkAutoSpaceUpdate)

    ON_COMMAND(ID_JSHARK_FREQ_REC, OnJSharkRecFrequency)
    ON_UPDATE_COMMAND_UI(ID_JSHARK_FREQ_REC, OnJSharkRecFrequencyUpdate)

    ON_COMMAND(ID_JSHARK_FREQ_USE, OnJSharkUseFrequency)
    ON_UPDATE_COMMAND_UI(ID_JSHARK_FREQ_USE, OnJSharkUseFrequencyUpdate)
END_MESSAGE_MAP()


JSharkApp::JSharkApp()
{
}


BOOL JSharkApp::InitInstance()
{
	InitCommonControls();
	CWinApp::InitInstance();

	// OLE Cu܂B
	if (!AfxOleInit()) {
		AfxMessageBox(IDP_OLE_INIT_FAILED);
		return FALSE;
	}
	AfxEnableControlContainer();

    // TODO: {ꃂ[h 
    ::_tsetlocale(LC_ALL, _T("japanese"));

    // ------ W

	SetRegistryKey(_T("JShark"));

    // C EChE쐬ƂÃR[h͐Vt[ EChE IuWFNg쐬A
	// AvP[ṼC EChEɃZbg܂

	CMainFrame* pFrame = new CMainFrame;
    if (!pFrame) { return FALSE; }

	m_pMainWnd = pFrame;
	// t[\[X烍[hč쐬܂
	pFrame->LoadFrame(IDR_MAINFRAME,
		WS_OVERLAPPEDWINDOW | FWS_ADDTOTITLE, NULL,
		NULL);

    // t@Cǂݍ݂܂B
    CFreqDictionaryManager::Instance().LoadDictionary();

    // C EBhEꂽ̂ŁA\ƍXVs܂B
	pFrame->ShowWindow(SW_SHOW);
	pFrame->UpdateWindow();

	return TRUE;
}


int JSharkApp::ExitInstance()
{
    CFreqDictionaryManager::Instance().SaveDictionary();

    return CWinApp::ExitInstance();
}


CMainFrame* JSharkApp::GetMainFrame() const
{
    return static_cast<CMainFrame*>(m_pMainWnd);
}


void JSharkApp::OnAppAbout()
{
	CAboutDialog about;
 
    // wv\ĂƂ́AꎞIɃ^C}[Ƃ߂B
    GetMainFrame()->TimerStop();
    about.DoModal();
    GetMainFrame()->TimerStart();
}


void JSharkApp::OnJSharkExit()
{
    TRACE(_T("OnJSharkExit"));

    AfxGetMainWnd()->PostMessage(WM_CLOSE);
}


void JSharkApp::OnJSharkAutoSpace()
{
    bool b = CSharkKeyboardManager::Instance().GetAutoSpace();
    CSharkKeyboardManager::Instance().SetAutoSpace(!b);
}


void JSharkApp::OnJSharkAutoSpaceUpdate(CCmdUI* pCmdUI)
{
    bool b = CSharkKeyboardManager::Instance().GetAutoSpace();
    pCmdUI->SetCheck(b);
}


void JSharkApp::OnJSharkUseFrequency()
{
    bool b = CSharkKeyboardManager::Instance().GetUseFrequency();
    CSharkKeyboardManager::Instance().SetUseFrequency(!b);
}


void JSharkApp::OnJSharkUseFrequencyUpdate(CCmdUI* pCmdUI)
{
    bool b = CSharkKeyboardManager::Instance().GetUseFrequency();
    pCmdUI->SetCheck(b);
}


void JSharkApp::OnJSharkRecFrequency()
{
    bool b = CSharkKeyboardManager::Instance().GetRecFrequency();
    CSharkKeyboardManager::Instance().SetRecFrequency(!b);
}


void JSharkApp::OnJSharkRecFrequencyUpdate(CCmdUI* pCmdUI)
{
    bool b = CSharkKeyboardManager::Instance().GetRecFrequency();
    pCmdUI->SetCheck(b);
}


